@@ -1,10 +1,11 @@
-from typing import Sequence
+from typing import Sequence, Type
 from dataclasses import dataclass
-from datetime import datetime
+from datetime import date as Date, datetime
 from dateutil.relativedelta import relativedelta
+from abc import ABCMeta, abstractmethod
 from enum import Enum
 
-from squirrels import _utils
+from . import _utils as u
 
 
 class DayOfWeek(Enum):
@@ -31,13 +32,13 @@ class Month(Enum):
     December = 12
 
 
-@dataclass
-class DateModifier:
+class DateModifier(metaclass=ABCMeta):
     """
     Interface for all Date modification classes, and declares a "modify" method
     """
 
-    def modify(self, date: datetime) -> datetime:
+    @abstractmethod
+    def modify(self, date: Date) -> Date:
         """
         Method to be overwritten, modifies the input date
 
@@ -47,19 +48,21 @@ class DateModifier:
         Returns:
             The modified date.
         """
-        raise _utils.AbstractMethodCallError(self.__class__, "modify")
+        pass
+
+    def _get_date(self, datetype: Type, year: int, month: int, day: int) -> Date:
+        return datetype(year, month, day)
 
 
-@dataclass
 class _DayIdxOfCalendarUnit(DateModifier):
     """
     Interface for adjusting a date to some day of calendar unit
     """
-    idx: int
-
-    def __post_init__(self):
+    def __init__(self, idx: int) -> None:
+        super().__init__()
+        self.idx = idx
         if self.idx == 0:
-            raise _utils.ConfigurationError("For constructors of class names that start with DayIdxOf_, idx cannot be zero")
+            raise u.ConfigurationError(f"For constructors of class names that start with DayIdxOf_, idx cannot be zero")
         self.incr = self.idx - 1 if self.idx > 0 else self.idx
 
 
@@ -73,21 +76,23 @@ class DayIdxOfMonthsCycle(_DayIdxOfCalendarUnit):
         num_months_in_cycle: 2 for one 6th of year, 3 for Quarter, 4 for one 3rd of year, 6 for half year, 12 for full year. Must fit evenly in 12
         first_month_of_cycle: The first month of months cycle of year. Default is January
     """
-    num_months_in_cycle: int
-    first_month_of_cycle: Month = Month.January
+    _num_months_in_cycle: int
+    _first_month_of_cycle: Month
 
-    def __post_init__(self):
-        super().__post_init__()
-        if 12 % self.num_months_in_cycle != 0:
-            raise _utils.ConfigurationError("Value X must fit evenly in 12")
-        self.first_month_of_first_cycle = (self.first_month_of_cycle.value - 1) % self.num_months_in_cycle + 1
+    def __init__(self, idx: int, num_months_in_cycle: int, first_month_of_cycle: Month = Month.January) -> None:
+        super().__init__(idx)
+        self._num_months_in_cycle = num_months_in_cycle
+        self._first_month_of_cycle = first_month_of_cycle
+        if 12 % self._num_months_in_cycle != 0:
+            raise u.ConfigurationError(f"Value X must fit evenly in 12")
+        self.first_month_of_first_cycle = (self._first_month_of_cycle.value - 1) % self._num_months_in_cycle + 1
 
-    def modify(self, date: datetime) -> datetime:
-        current_cycle = (date.month - self.first_month_of_first_cycle) % 12 // self.num_months_in_cycle
-        first_month_of_curr_cycle = current_cycle * self.num_months_in_cycle + self.first_month_of_first_cycle
+    def modify(self, date: Date) -> Date:
+        current_cycle = (date.month - self.first_month_of_first_cycle) % 12 // self._num_months_in_cycle
+        first_month_of_curr_cycle = current_cycle * self._num_months_in_cycle + self.first_month_of_first_cycle
         year = date.year if date.month >= first_month_of_curr_cycle else date.year - 1
-        first_day = datetime(year, first_month_of_curr_cycle, 1)
-        ref_date = first_day if self.idx > 0 else first_day + relativedelta(months=self.num_months_in_cycle)
+        first_day = self._get_date(type(date), year, first_month_of_curr_cycle, 1)
+        ref_date = first_day if self.idx > 0 else first_day + relativedelta(months=self._num_months_in_cycle)
         return ref_date + relativedelta(days=self.incr)
 
 
@@ -128,8 +133,11 @@ class DayIdxOfMonth(_DayIdxOfCalendarUnit):
         idx: 1 for first, 2 for second, etc. Or, -1 for last, -2 for second last, etc. Must not be 0
     """
 
-    def modify(self, date: datetime) -> datetime:
-        first_day = datetime(date.year, date.month, 1)
+    def __init__(self, idx: int) -> None:
+        super().__init__(idx)
+
+    def modify(self, date: Date) -> Date:
+        first_day = self._get_date(type(date), date.year, date.month, 1)
         ref_date = first_day if self.idx > 0 else first_day + relativedelta(months=1)
         return ref_date + relativedelta(days=self.incr)
 
@@ -143,24 +151,26 @@ class DayIdxOfWeek(_DayIdxOfCalendarUnit):
         idx: 1 for first, 2 for second, etc. Or, -1 for last, -2 for second last, etc. Must not be 0
         first_day_of_week: The day of week identified as the "first". Default is Monday
     """
-    first_day_of_week: DayOfWeek = DayOfWeek.Monday
+    _first_day_of_week: DayOfWeek
 
-    def __post_init__(self):
-        super().__post_init__()
-        self.first_dow_num = self.first_day_of_week.value
+    def __init__(self, idx: int, first_day_of_week: DayOfWeek = DayOfWeek.Monday) -> None:
+        super().__init__(idx)
+        self._first_day_of_week = first_day_of_week
+        self.first_dow_num = self._first_day_of_week.value
     
-    def modify(self, date: datetime) -> datetime:
+    def modify(self, date: Date) -> Date:
         distance_from_first_day = (1 + date.weekday() - self.first_dow_num) % 7
         total_incr = -distance_from_first_day + (7 if self.idx < 0 else 0) + self.incr
         return date + relativedelta(days=total_incr)
 
 
-@dataclass
 class _OffsetUnits(DateModifier):
     """
     Abstract DateModifier class to offset an input date by some number of some calendar unit
     """
-    offset: int
+    def __init__(self, offset: int) -> None:
+        super().__init__()
+        self.offset = offset
 
 
 @dataclass
@@ -172,7 +182,10 @@ class OffsetYears(_OffsetUnits):
         offset: The number of years to offset the input date.
     """
 
-    def modify(self, date: datetime) -> datetime:
+    def __init__(self, offset: int) -> None:
+        super().__init__(offset)
+
+    def modify(self, date: Date) -> Date:
         return date + relativedelta(years=self.offset)
     
 
@@ -185,7 +198,10 @@ class OffsetMonths(_OffsetUnits):
         offset: The number of months to offset the input date.
     """
 
-    def modify(self, date: datetime) -> datetime:
+    def __init__(self, offset: int) -> None:
+        super().__init__(offset)
+
+    def modify(self, date: Date) -> Date:
         return date + relativedelta(months=self.offset)
 
 
@@ -198,7 +214,10 @@ class OffsetWeeks(_OffsetUnits):
         offset: The number of weeks to offset the input date.
     """
 
-    def modify(self, date: datetime) -> datetime:
+    def __init__(self, offset: int) -> None:
+        super().__init__(offset)
+
+    def modify(self, date: Date) -> Date:
         return date + relativedelta(weeks=self.offset)
 
 
@@ -211,7 +230,10 @@ class OffsetDays(_OffsetUnits):
         offset: The number of days to offset the input date.
     """
 
-    def modify(self, date: datetime) -> datetime:
+    def __init__(self, offset: int) -> None:
+        super().__init__(offset)
+
+    def modify(self, date: Date) -> Date:
         return date + relativedelta(days=self.offset)
 
 
@@ -223,13 +245,14 @@ class DateModPipeline(DateModifier):
     Attributes:
         modifiers: The list of DateModifier's to apply in sequence.
     """
-    date_modifiers: Sequence[DateModifier]
+    _date_modifiers: Sequence[DateModifier]
 
-    def __post_init__(self):
-        self.date_modifiers = tuple(self.date_modifiers)
+    def __init__(self, date_modifiers: Sequence[DateModifier]) -> None:
+        super().__init__()
+        self._date_modifiers = tuple(date_modifiers)
     
-    def modify(self, date: datetime) -> datetime:
-        for modifier in self.date_modifiers:
+    def modify(self, date: Date) -> Date:
+        for modifier in self._date_modifiers:
             date = modifier.modify(date)
         return date
     
@@ -244,7 +267,7 @@ class DateModPipeline(DateModifier):
         Returns:
             A new sequence of DateModifier
         """
-        joined_modifiers = self.date_modifiers + tuple(date_modifiers)
+        joined_modifiers = self._date_modifiers + tuple(date_modifiers)
         return joined_modifiers
 
     def with_more_modifiers(self, date_modifiers: Sequence[DateModifier]):
@@ -260,13 +283,13 @@ class DateModPipeline(DateModifier):
         joined_modifiers = self.get_joined_modifiers(date_modifiers)
         return DateModPipeline(joined_modifiers)
     
-    def get_date_list(self, start_date: datetime, step: _OffsetUnits) -> Sequence[datetime]:
+    def get_date_list(self, start_date: Date, step: _OffsetUnits) -> Sequence[Date]:
         """
         This method modifies the input date, and returns all dates from the input date to the modified date, 
         incremented by a DateModifier step.
 
         If the step is positive and start date is less than end date, then it'll return an increasing list of
-        dates starting from the start date. If the step is positive and start date is less than end date, 
+        dates starting from the start date. If the step is negative and start date is greater than end date, 
         then it'll return a decreasing list of dates starting from the start date. Otherwise, an empty list
         is returned.
 
@@ -278,7 +301,7 @@ class DateModPipeline(DateModifier):
             A list of datetime objects
         """
         if step.offset == 0:
-            raise _utils.ConfigurationError("The length of 'step' must not be zero")
+            raise u.ConfigurationError(f"The length of 'step' must not be zero")
         
         output = []
         end_date = self.modify(start_date)
@@ -291,16 +314,16 @@ class DateModPipeline(DateModifier):
         return output
 
 
-@dataclass
-class _DateRepresentationModifier:
+class _DateRepresentationModifier(metaclass=ABCMeta):
     """
     Abstract class for modifying other representations of dates (such as string or unix timestemp)
     """
     def __init__(self, date_modifiers: Sequence[DateModifier]):
         self.date_modifier = DateModPipeline(date_modifiers)
 
+    @abstractmethod
     def with_more_modifiers(self, date_modifiers: Sequence[DateModifier]):
-        raise _utils.AbstractMethodCallError(self.__class__, "with_more_modifiers")
+        pass
 
 
 @dataclass
@@ -312,9 +335,12 @@ class DateStringModifier(_DateRepresentationModifier):
         date_modifier: The DateModifier to apply on datetime objects
         date_format: Format of the output date string. Default is '%Y-%m-%d'
     """
+    _date_modifiers: Sequence[DateModifier]
+    _date_format: str
+
     def __init__(self, date_modifiers: Sequence[DateModifier], date_format: str = '%Y-%m-%d'):
         super().__init__(date_modifiers)
-        self.date_format = date_format
+        self._date_format = date_format
 
     def with_more_modifiers(self, date_modifiers: Sequence[DateModifier]):
         """
@@ -327,11 +353,11 @@ class DateStringModifier(_DateRepresentationModifier):
             A new DateStringModifier
         """
         joined_modifiers = self.date_modifier.get_joined_modifiers(date_modifiers)
-        return DateStringModifier(joined_modifiers, self.date_format)
+        return DateStringModifier(joined_modifiers, self._date_format)
     
-    def _get_input_date_obj(self, date_str: str, input_format: str = None) -> datetime:
-        input_format = self.date_format if input_format is None else input_format
-        return datetime.strptime(date_str, input_format)
+    def _get_input_date_obj(self, date_str: str, input_format: str = None) -> Date:
+        input_format = self._date_format if input_format is None else input_format
+        return datetime.strptime(date_str, input_format).date()
 
     def modify(self, date_str: str, input_format: str = None) -> str:
         """
@@ -345,7 +371,7 @@ class DateStringModifier(_DateRepresentationModifier):
             The resulting date string
         """
         date_obj = self._get_input_date_obj(date_str, input_format)
-        return self.date_modifier.modify(date_obj).strftime(self.date_format)
+        return self.date_modifier.modify(date_obj).strftime(self._date_format)
     
     def get_date_list(self, start_date_str: str, step: DateModifier, input_format: str = None) -> Sequence[str]:
         """
@@ -353,7 +379,7 @@ class DateStringModifier(_DateRepresentationModifier):
         to the modified date, incremented by a DateModifier step.
 
         If the step is positive and start date is less than end date, then it'll return an increasing list of
-        dates starting from the start date. If the step is positive and start date is less than end date, 
+        dates starting from the start date. If the step is negative and start date is greater than end date, 
         then it'll return a decreasing list of dates starting from the start date. Otherwise, an empty list
         is returned.
 
@@ -367,7 +393,7 @@ class DateStringModifier(_DateRepresentationModifier):
         """
         curr_date = self._get_input_date_obj(start_date_str, input_format)
         output = self.date_modifier.get_date_list(curr_date, step)
-        return [x.strftime(self.date_format) for x in output]
+        return [x.strftime(self._date_format) for x in output]
 
 
 @dataclass
@@ -379,6 +405,8 @@ class TimestampModifier(_DateRepresentationModifier):
         date_modifier: The DateModifier to apply on datetime objects
         date_format: Format of the date string. Default is '%Y-%m-%d'
     """
+    _date_modifiers: Sequence[DateModifier]
+
     def __init__(self, date_modifiers: Sequence[DateModifier]):
         super().__init__(date_modifiers)
 
@@ -406,7 +434,8 @@ class TimestampModifier(_DateRepresentationModifier):
             The resulting timestamp
         """
         date_obj = datetime.fromtimestamp(timestamp)
-        return self.date_modifier.modify(date_obj).timestamp()
+        modified_date: datetime = self.date_modifier.modify(date_obj)
+        return modified_date.timestamp()
     
     def get_date_list(self, start_timestamp: float, step: DateModifier) -> Sequence[float]:
         """
@@ -414,7 +443,7 @@ class TimestampModifier(_DateRepresentationModifier):
         to the modified date, incremented by a DateModifier step.
 
         If the step is positive and start date is less than end date, then it'll return an increasing list of
-        dates starting from the start date. If the step is positive and start date is less than end date, 
+        dates starting from the start date. If the step is negative and start date is greater than end date, 
         then it'll return a decreasing list of dates starting from the start date. Otherwise, an empty list
         is returned.
 
@@ -426,5 +455,5 @@ class TimestampModifier(_DateRepresentationModifier):
             A list of timestamp as floats
         """
         curr_date = datetime.fromtimestamp(start_timestamp)
-        output = self.date_modifier.get_date_list(curr_date, step)
+        output: Sequence[datetime] = self.date_modifier.get_date_list(curr_date, step)
         return [x.timestamp() for x in output]
